
#include <math.h>
#include "Python.h"
#include "hull.h"

static long site_index;
static long num_sites;
static site site_buf;
static site *site_list;
static int num_dims;
static PyObject* result_list;

FILE *DFILE = stderr;

static long
site_numm(site s)
{
  if(s >= site_buf && s < site_buf + site_index*num_dims)
    return (s - site_buf) / num_dims;
  return -1;
}

static site
get_next_site(void)
{
  if(site_index >= num_sites)
    return NULL;
  return site_list[site_index++];
}

static void
vlist_cvt(point *v, int vdim, FILE *Fin, int amble)
{
	int j;

	if (v)
	{
	  PyObject* tuple = PyTuple_New(vdim);
	  if(tuple)
	  {
	    for (j = 0; j < vdim; ++j)
	      PyTuple_SET_ITEM(tuple, j, PyInt_FromLong(site_numm(v[j])));
	    PyList_Append(result_list, tuple);
	  }
	}
}

static PyObject *
make_py_num(long i)
{
  PyObject* numberi = PyInt_FromLong(i);
  Py_DECREF(numberi);
  return numberi;
}

static PyObject*
compute_hull(PyObject *args, int do_delaunay)
{
	simplex *root;
	int i;
	int j;
	PyObject* py_points;
	PyObject* py;
	result_list = NULL;
	if (!PyArg_ParseTuple(args, "Od", &py_points, &mult_up))
	  return NULL;
	num_sites = PyObject_Length(py_points);
	if(num_sites <= 0)
	{
	  PyErr_SetString(PyExc_ValueError,
			  "len(py_points) didn't return positive number");
	  return NULL;
	}
	py = PyObject_GetItem(py_points, make_py_num(0));
	if(!py)
	{
	  PyErr_SetString(PyExc_IndexError, "py_points[0] failed");
	  return NULL;
	}
	num_dims = PyObject_Length(py);
	if (num_dims > MAXDIM)
	{
	  char buf[512];
	  sprintf(buf, "Number of dimensions (%d) exceeds MAXDIM (%d)",
		  num_dims, MAXDIM);
	  PyErr_SetString(PyExc_ValueError, buf);
	  return NULL;
	}
	point_size = site_size = sizeof(Coord)*num_dims;
	if(site_list)
	{
	  PyErr_SetString(PyExc_RuntimeError, "compute_hull entered 2nd time before first finished");
	  return NULL;
	}
	site_buf = (site)malloc(num_sites * site_size);
	site_list = (site*)malloc(num_sites * sizeof(*site_list));
	for(j = 0; j < num_sites; ++j)
	{
	  py = PyObject_GetItem(py_points, make_py_num(j));
	  if(!py)
	  {
	    char buf[512];
	    sprintf(buf, "py_points[%d] failed", j);
	    PyErr_SetString(PyExc_IndexError, buf);
	    goto error;
	  }
	  site_list[j] = p = &site_buf[j*num_dims];
	  for(i = 0; i < num_dims; ++i)
	  {
	    PyObject *v = PyObject_GetItem(py, make_py_num(i));
	    if(!v)
	    {
	      char buf[512];
	      sprintf(buf, "py_points[%d][%d] failed", j, i);
	      PyErr_SetString(PyExc_IndexError, buf);
	      goto error;
	    }
	    if(!v)
	    {
	      char buf[512];
	      sprintf(buf, "py_points[%d][%d] cannot be converted to a float", j, i);
	      PyErr_SetString(PyExc_TypeError, buf);
	      goto error;
	    }
	    p[i] = floor(mult_up*PyFloat_AsDouble(v)+0.5);
	    mins[i] = (mins[i]<p[i]) ? mins[i] : p[i];
	    maxs[i] = (maxs[i]>p[i]) ? maxs[i] : p[i];
	  }
	}
	site_index = 0;

	root = build_convex_hull(get_next_site, site_numm,
				 num_dims, do_delaunay);

	result_list = PyList_New(0);

	if(root)
	{
	  visit_func *pr = facets_print;
	  out_func* mof = vlist_cvt;
	  mof(0,0,stdout,-1);
	  pr(0, mof);
	  visit_hull(root, pr);
	  mof(0,0,stdout,1);
	}

	if(1) free_hull_storage();/* ?? when to do this? */
 error:
	free(site_buf);
	site_buf = NULL;
	free(site_list);
	site_list = NULL;
	return result_list;
}

static PyObject*
Delaunay(PyObject *dummy, PyObject *args)
{
  return compute_hull(args, 1);
}

static PyObject*
ConvexHull(PyObject *dummy, PyObject *args)
{
  return compute_hull(args, 0);
}

/*
 * List of functions defined in the module
 */

static PyMethodDef hull_methods[] = {
  {"Delaunay", Delaunay, METH_VARARGS},
  {"ConvexHull", ConvexHull, METH_VARARGS},
  {NULL, NULL}		/* sentinel */
};

/* Initialization function for the module */

void
inithull()
{
  PyObject *m;

  /* Create the module and add the functions */
  m = Py_InitModule("hull", hull_methods);
  
  /* Check for errors */
  if (PyErr_Occurred())
    Py_FatalError("can't initialize module hull");
}
