# This module implements a DCD reader/writer
#
# Written by Lutz Ehrlich
# Adapted to MMTK conventions by Konrad Hinsen
# last revision: 1998-2-25
#
import _DCD
import ConfigIO, Trajectory, Units
import copy, Numeric, operator


class DCDReader(Trajectory.TrajectoryGenerator):

    default_options = {'trajectory': None, 'log': None}

    available_data = ['configuration', 'time']

    def __init__(self, universe, **options):
	Trajectory.TrajectoryGenerator.__init__(self, universe, options)

    def __call__(self, **options):
	self.setCallOptions(options)
	configuration = self.universe.configuration()
	_DCD.readDCD(self.universe, configuration.array,
		     [self.getOption('trajectory'), self.getOption('log')],
		     self.getOption('dcd_file'))


        
def writeDCD(conf_list, dcd_file_name, atom_order=None, delta_t=0.1):
    universe = conf_list[0].universe
    natoms = universe.numberOfCartesianCoordinates()
    if atom_order is None:
	atom_order = Numeric.arrayrange(natoms)
    else:
	atom_order = Numeric.array(atom_order)

    i_start = 0       # always start at frame 0
    n_savc  = 1       # save every frame
    fd = _DCD.writeOpenDCD(dcd_file_name, natoms, len(conf_list), i_start,
			   n_savc, delta_t )
    for conf in conf_list:
        _DCD.writeDCDStep(fd, conf.array, atom_order)
    _DCD.writeCloseDCD(fd)


def writeDCDPDB(conf_list, dcd_file_name, pdb_file_name, delta_t=0.1):
    universe = conf_list[0].universe
    universe.configuration()
    pdb = ConfigIO.PDBOutputFile(pdb_file_name)
    pdb.write(universe, conf_list[0])
    indices = map(lambda a: a.index, pdb.atom_sequence)
    pdb.close()
    del pdb
    writeDCD(conf_list, dcd_file_name, indices, delta_t)
