from mmtk import *

# Define system
world = InfiniteUniverse(AmberForceField())
world.protein = Protein('bala1')

# Initialize velocities
world.initializeVelocitiesToTemperature(50.*Units.K)
print 'Temperature: ', world.temperature()
print 'Momentum: ', world.momentum()
print 'Angular momentum: ', world.angularMomentum()

# Create integrator
integrator = VelocityVerletIntegrator(world, log=None, trajectory=None)

# Equilibration
integrator(delta_t=1.*Units.fs, steps=1000, log=50,
	   options=[Heater(50.*Units.K, 300.*Units.K, 0.5*Units.K/Units.fs),
		    (TranslationRemover(), 50),
		    (RotationRemover(), 50)])

# "Production" run
trajectory = Trajectory(world, "bala1.nc", "w", "A simple test case")
integrator(delta_t=1.*Units.fs, steps=100,
	   log=(0, None, 50, stdout, ("time", "energy")),
	   trajectory=(0, None, 1, trajectory, ("time", "energy", "temperature",
						"configuration")),
	   options=[(TranslationRemover(), 50),
		    (RotationRemover(), 50)])
trajectory.close()

# Print information about the trajectory file
print trajectoryInfo('bala1.nc')

# Reopen trajectory file
trajectory = Trajectory(world, "bala1.nc", "r")

# Read step 10 and display configuration
step10 = trajectory[10]
view(world, step10['configuration'])

# Print the kinetic energy along the trajectory
print "Kinetic energy along trajectory:"
print trajectory.kinetic_energy

trajectory.close()
