# This module implements a class that represents a way in which a Group
# can be attached to another Group
#
# Written by Peter McCluskey (pcm@rahul.net)
#

import Database, Bonds
import DictWithDefault, types
import Utility, Units

debugH = 0
def setDebug(level):
    debugH = level

class Hook:
    def __init__(self, blueprint, _memo = None, group = None):
	if blueprint is not None:
	    if type(blueprint) is type(()):
		self._group = blueprint[0]
		self._danglingBonds = blueprint[1]
	    else:
		if group is not None:
		    self._group = group
		else:
		    self._group = Database.instantiate(blueprint.group, _memo)
		self._danglingBonds = Database.instantiate(blueprint.danglingbonds, _memo)
	    if isHook(self._danglingBonds):
		self._danglingBonds = self._danglingBonds._danglingBonds[:]
	    try:
		n = len(self)
	    except TypeError:
		Utility.warning('Hook defined without list of dangling bonds')
	    else:
		if n < 1:
		    Utility.warning('Hook defined without any dangling bonds')
		elif not Bonds.isBond(self._danglingBonds[0]):
		    raise TypeError,'Hook dangling bond list contains something that isnt a Bond'

    is_hook = 1

    def __len__(self):
	return len(self._danglingBonds)

    def __getitem__(self, i):
	return self._danglingBonds[i]
    def __setitem__(self, i, val):
	self._danglingBonds[i] = val
    def __getslice__(self, first, last):
	return Hook((self._group, self._danglingBonds[first:last]))

    def setGroup(self, group):
	self._group = group

    def getGroup(self):
	return self._group

    def getBonds(self):
	return self._danglingBonds

    def verify(self, tag = ''):
	print 'Hooks.verify',tag,hex(id(self)),len(self._danglingBonds)
	if self._group is not None:
	    for db in self._danglingBonds:
		    if not db in self._group.danglingbonds:
			try:
			    print hex(id(db)),db,db.constraint.angle/Units.deg
			except: pass
			print 'not in'
			for b in self._group.danglingbonds:
			    try:
				print hex(id(b)),b,b.constraint.angle/Units.deg
			    except: print hex(id(b)),b,'{no angle}'
			if hasattr(self._group, 'subgroups'):
			    for grp in self._group.subgroups:
				if b in grp.danglingbonds:
				    print 'is in subgroup'
			#raise ValueError

    def matchList(self, dblist):
	if len(dblist) != len(self._danglingBonds):
	    return 0
	for i in range(len(self._danglingBonds)):
	    if not self._danglingBonds[i] is dblist[i]:
		return 0
	return 1

    def __cmp__(self, other):
	if len(self) != len(other):
	    return 1
	for i in range(len(self)):
	    if not self[i] is other[i]:
		return 1
	if not self._group is other._group:
	    return 1
	return 0

    def Label(self, basename):
	for i in range(len(self)):
	    db = self[i]
	    if not db.HookLabel():
		db.setHookLabel(basename + '[' + `i` + ']')
	    #if db not in self._group.danglingbonds:
		#print 'Label error',db
	    #print ' '*len(basename),'labelDanglingBonds',hex(id(db)),hex(id(db.a1)),db.HookLabel(),db.a1.fullName()[-4:]

    def __repr__(self):
	str = 'Hook('
	for db in self._danglingBonds:
	    str = str + db.a1.fullName() + ','
	return str + ')'
    __str__ = __repr__

    blueprintclass = Database.BlueprintHook

Database.registerInstanceClass(Hook.blueprintclass, Hook)

def isHook(obj):
    return hasattr(obj, 'is_hook')

class HookList:

    is_hooklist = 1

    def __init__(self, list):
	if(isHookList(list)):
	    list = list.data
	self.data = list
	if list:
	    assert(type(list[0]) != type([]))
	if 0:
	    try:
		tmp = list[0].a1
	    except:
		print 'Hook.__init__',hex(id(self)),`list`[:64]

    def Label(self, basename):
	for i in range(len(self)):
	    #print 'HookList.Label',i,self.data[i]
	    self.data[i].Label(basename + ('[%d]' % i))

    def __add__(self, other):
	return HookList(self.data + other)
    __radd__ = __add__

    def __repr__(self):
	return repr(self.data)
    def __getitem__(self, i):
	return self.data[i]
    def __len__(self):
	return len(self.data)
    def append(self, hook):
	assert(type(hook) != type([]) and hook is not None)
	self.data.append(hook)
    def __setitem__(self, i, val):
	assert(type(val) != type([]) and val is not None)
	self.data[i] = val
    def __getslice__(self, first, last):
	return HookList(self.data[first:last])
    def index(self, object):
	return self.data.index(object)
    def remove(self, object):
	self.data.remove(object)

    def verify(self):
	for h in self.data:
	    h.verify()

def isHookList(obj):
    return hasattr(obj, 'is_hooklist')

def mergeHooklist(hlist, new_hook_dict = None):
    dblist = []
    for h in hlist:
	if new_hook_dict is not None:
	    if not new_hook_dict[id(h)]:
		print '_mergeHooklist cant find',h,'\nhlist',hlist
	    new_h = new_hook_dict[id(h)]
	    if new_h is not None:
		h = new_h
	dblist = dblist + h.getBonds()
    return dblist


