# This module generates various lattices.
#
# Written by Konrad Hinsen
# last revision: 1996-10-2
#

from Vector import Vector, ex, ey, ez

#
# Lattice base class
#
class Lattice:

    def __init__(self, function):
	if function is not None:
	    self.elements = map(function, self.elements)

    def __getitem__(self, item):
	return self.elements[item]

    def __setitem__(self, item, value):
	self.elements[item] = value

    def __len__(self):
	return len(self.elements)

#
# General rhombic lattice
#
class RhombicLattice(Lattice):

    def __init__(self, elementary_cell, lattice_vectors, cells, function=None):
	if len(lattice_vectors) != len(cells):
	    raise TypeError, 'Inconsistent dimension specification'
	self.dimension = len(lattice_vectors)
	self.elements = []
	self.makeLattice(elementary_cell, lattice_vectors, cells, Vector(0,0,0))
	Lattice.__init__(self, function)

    def makeLattice(self, elementary_cell, lattice_vectors, cells, base):
	if len(cells) == 0:
	    for p in elementary_cell:
		self.elements.append(p+base)
	else:
	    for i in range(cells[0]):
		self.makeLattice(elementary_cell, lattice_vectors[1:],
				 cells[1:], base+i*lattice_vectors[0])
	    
#
# Bravais lattice
#
class BravaisLattice(RhombicLattice):

    def __init__(self, lattice_vectors, cells, function=None):
	cell = [Vector(0,0,0)]
	RhombicLattice.__init__(self, cell, lattice_vectors, cells, function)

#
# Simple cubic lattice
#
class SCLattice(BravaisLattice):

    def __init__(self, cellsize, cells, function=None):
	lattice_vectors = (cellsize*ex, cellsize*ey, cellsize*ez)
	if type(cells) != type(()):
	    cells = 3*(cells,)
	BravaisLattice.__init__(self, lattice_vectors, cells, function)
