# Functions for finding random points and orientations.
#
# Written by: Konrad Hinsen <hinsen@ibs.ibs.fr>
# Last revision: 1997-5-5
# 

import random, umath
from Vector import Vector
from RandomArray import uniform
from Numeric import dot
from Transformation import Rotation
import Units

#
# Initialize random number generator to a fixed value
# (default is to initialize from current time)
#
from whrandom import seed
seed(1, 1, 1)
del seed

#
# Initialize from current time
#
def initializeRandomNumbersFromTime():
    from whrandom import seed
    seed(0, 0, 0)

#
# Random point in a rectangular box centered around the origin
#
def randomPointInBox(a, b = None, c = None):
    if b is None: b = a
    if c is None: c = a
    x = uniform(-0.5*a, 0.5*a)
    y = uniform(-0.5*b, 0.5*b)
    z = uniform(-0.5*c, 0.5*c)
    return Vector(x, y, z)

#
# Random point in a sphere around the origin.
#
def randomPointInSphere(r):
    rsq = r*r
    while 1:
	x = uniform(-r, r, (3,))
	if dot(x, x) < rsq: break
    return Vector(x)

#
# Random direction (unit vector).
#
def randomDirection():
    r = randomPointInSphere(1.)
    return r.normal()

#
# Random rotation.
#
def randomRotation(max_angle = umath.pi):
    return Rotation(randomDirection(), uniform(-max_angle, max_angle))

#
# Random velocity (gaussian)
#
def randomVelocity(temperature, mass):
    sigma = umath.sqrt((temperature*Units.k_B)/(mass*Units.amu))
    return Vector(random.normalvariate(0., sigma),
		  random.normalvariate(0., sigma),
		  random.normalvariate(0., sigma))
