# Python wrapper for Roger Sayle's RasMol
# written by Peter McCluskey (pcm@rahul.net)
#
# polyColor doesn't work yet, and it core dumps if you try to run
# a 2nd Rasmol in a given python session. Select functions probably don't work.
#

import _rasmol

class RasMol:
    def __init__(self, compositeObj, labelFullName = 0, displayLabels = 0,\
		 polyColor = (255, 255, 255)):
	self.destroyed = 0
	self._poly_ids = {}
	self.labelFullName = labelFullName
	_rasmol.setLabelFullNameFlag(labelFullName)
	_rasmol.displayLabels(displayLabels)
	_rasmol.__init__(compositeObj)
	_rasmol.RefreshScreen()
	_rasmol.SetPolyColor(polyColor)

    def __del__(self):
	for key in self._poly_ids.keys():
	    _rasmol.RemovePoly(key)
	if not self.destroyed:
	    _rasmol.__del__()

    def Interact(self):
	cmd = _rasmol.Interact()
	if cmd is None:
	    self.destroyed = 1

    def RefreshScreen(self):
	_rasmol.RefreshScreen()

	# selects atoms for which selected_atoms_dict.has_key(atom.index)
	# is true, deselects rest
    def SelectAtoms(self, selected_atoms_dict):
	return _rasmol.SelectAtoms(selected_atoms_dict)

	# selects atoms for which selected_bonds_dict.has_key((bond.a1.index,
	# bond.a2.index)) is true, deselects rest
    def SelectBonds(self, selected_bonds_dict):
	return _rasmol.SelectBonds(selected_bonds_dict)

    def AddPolygon(self, coordList):
	assert(len(coordList) > 0)
	if len(coordList) > _rasmol.MaxPolyCoords():
	    msg = 'coordinate list length (%d) exceeds polygon limit (%d)'\
		  % (len(coordList), _rasmol.MaxPolyCoords())
	    raise ValueError, msg
	poly_id = _rasmol.AddPolygon(coordList)
	self._poly_ids[poly_id] = coordList
	return poly_id

    def AddPolyLines(self, coordList):
	assert(len(coordList) > 0)
	batch_size = _rasmol.MaxPolyCoords()
	id_list = []
	while coordList:
	    batch = coordList[0:batch_size]
	    poly_id = _rasmol.AddPolyLines(batch)
	    self._poly_ids[poly_id] = batch
	    id_list.append(poly_id)
	    coordList = coordList[batch_size:]
	return id_list

    def RemovePoly(self, poly_id):
	if type(poly_id) == type([]):
	    for id1 in poly_id:
		self.RemovePoly(id1)
	else:
	    _rasmol.RemovePoly(poly_id)
	    del self._poly_ids[poly_id]
