# This module handles the skeleton descriptions stored in trajectory files.
#
# Written by Konrad Hinsen
# last revision: 1997-6-27
#

import mmtk

#
# Atoms
#
class A:

    def __init__(self, name, index, type = None):
	self.name = name
	self.index = index
	self.type = type

    def make(self, array = None):
	atom =  mmtk.Atom(self.type, name = self.name)
	atom.setArray(array, [self.index])
	return atom

    def assignIndex(self, atom, array):
	atom.setArray(array, [self.index])

#
# Composite chemical objects
#
class Composite:

    def __init__(self, name, list, type = None, **kwargs):
	self.name = name
	self.list = list
	self.type = type
	self.kwargs = kwargs

    def make(self, array = None):
	object = self._class(self.type, name=self.name)
	for sub in self.list:
	    sub.assignIndex(getattr(object, sub.name), array)
	return object

    def assignIndex(self, object, array):
	for sub in self.list:
	    sub.assignIndex(getattr(object, sub.name), array)

class G(Composite):
    pass

class M(Composite):
    _class = mmtk.Molecule

class C(Composite):
    _class = mmtk.Complex

class X(Composite):
    _class = mmtk.Crystal

class S(Composite):

    def make(self, array = None):
	n_residues = len(self.type)/3
	residues = map(lambda i, s = self.type: s[3*i:3*i+3], range(n_residues))
	self.kwargs['name'] = self.name
	chain = apply(mmtk.PeptideChain, (residues,), self.kwargs)
	for i in range(len(self.list)):
	    self.list[i].assignIndex(chain[i], array)
	return chain

#
# Collections and universes
#
class c:

    def __init__(self, creation, objects):
	self.creation = creation
	self.objects = objects

    def make(self, array = None):
	local = {}
	collection = eval(self.creation, vars(mmtk), local)
	for o in self.objects:
	    collection.addObject(o.make(array))
	return collection

#
# Objects constructed from a list of other objects (e.g. proteins)
#
class l:

    def __init__(self, class_name, name, objects):
	self.class_name = class_name
	self.objects = objects
	self.name = name

    def make(self, array = None):
	return getattr(mmtk, self.class_name) \
	       (map(lambda o, a=array: o.make(a), self.objects),
		name = self.name)
