# This module contains code for solvation
#
# Written by Konrad Hinsen
# last revision: 1997-8-28
#

import ChemicalObjects, Units, Universe
from Minimization import SteepestDescentMinimizer
from Dynamics import VelocityVerletIntegrator, VelocityScaler, Heater, \
                     TranslationRemover, RotationRemover
import Vector
import copy, sys

#
# Add solvent to a universe containing solute molecules.
# This is a method to be added to class Universe.Universe.
#
def addSolvent(self, solvent, density, temperature=300.*Units.K):

    # Calculate number of solvent molecules and universe size
    if type(solvent) == type(''):
	solvent = ChemicalObjects.Molecule(solvent)
    cell_volume = self.cellVolume()
    if cell_volume is None:
	raise TypeError, "can't add solvent to an infinite universe"
    solute = copy.copy(self._objects)
    solute_volume = 0.
    excluded_regions = []
    for o in solute:
	solute_volume = solute_volume + self.surfaceAndVolume()[1]
	excluded_regions.append(o.boundingSphere())
    n_solvent = int(round(density*(cell_volume-solute_volume)/solvent.mass()))
    solvent_volume = n_solvent*solvent.mass()/density
    cell_volume = solvent_volume + solute_volume
    self.translateBy(-solute.position())
    self.scaleSize((cell_volume/self.cellVolume())**(1./3.))

    # Scale up the universe and add solvent molecules at random positions
    scale_factor = 5.
    self.scaleSize(scale_factor)
    for i in range(n_solvent):
	m = copy.copy(solvent)
	m.translateTo(self.randomPoint())
	while 1:
	    s = m.boundingSphere()
	    collision = 0
	    for region in excluded_regions:
		if (s.center-region.center).length() < s.radius+region.radius:
		    collision = 1
		    break
	    if not collision:
		break
	    m.translateTo(self.randomPoint())
	self.addObject(m)
	excluded_regions.append(s)

    # Do some minimization and equilibration
    self.initializeVelocitiesToTemperature(temperature)
    minimizer = SteepestDescentMinimizer(self, log=None, trajectory=None,
					 step_size = 0.05*Units.Ang)
    integrator = VelocityVerletIntegrator(self, log=None, trajectory=None,
		      options=[VelocityScaler(temperature, 0.05*temperature),
			       (TranslationRemover(), 50),
			       (RotationRemover(), 50)])
    for i in range(5):
	minimizer(steps = 10)
    integrator(steps = 100)
    self.translateBy(-solute.position())

    # Scale down the system in small steps
    while scale_factor > 1.:
	step_factor = max(0.95, 1./scale_factor)
	for object in self:
	    object.translateTo(step_factor*object.position())
	self.scaleSize(step_factor)
	scale_factor = scale_factor*step_factor
	for i in range(5):
	    minimizer(steps = 10)
	integrator(steps = 100)
	self.translateBy(-solute.position())

Universe.Universe.addSolvent = addSolvent
