# This module contains useful functions that are needed in various
# places.
#
# Written by Konrad Hinsen
# last revision: 1998-6-22
#

import Database
import os, string, sys, types
import Numeric

# Constants

undefined_limit = 1.e30
undefined = 10.*undefined_limit

# Error

MMTKError = 'MMTKError'

#
# This function substitutes all references to one object by references to
# another object in a given object recursively. The  substitution is
# specified by a dictionary.
# Attention: For some object types, substitution is destructive!
#
def substitute(object, *exchange):
    if len(exchange) == 1:
	exchange = exchange[0]
    else:
	dict = {}
	map(lambda k, v, d=dict: _put(d, k, v), exchange[0], exchange[1])
	exchange = dict
    if type(object) == types.ListType:
	return map(lambda e, x=exchange: substitute(e, x), object)
    if type(object) == types.TupleType:
	return tuple(map(lambda e, x=exchange: substitute(e, x), object))
    elif type(object) == types.DictionaryType:
	newdict = {}
	for key, value in object.items():
	    newdict[substitute(key, exchange)] = substitute(value, exchange)
	return newdict
    elif type(object) == types.InstanceType and hasattr(object, 'substitute'):
	for attr in dir(object):
	    setattr(object, attr, substitute(getattr(object, attr), exchange))
	return object
    elif exchange.has_key(object):
	return exchange[object]
    else:
	return object

def _put(dict, key, value):
    dict[key] = value

#
# Return a unique attribute name
#
_unique_attributes = 0
def uniqueAttribute():
    global _unique_attributes
    _unique_attributes = (_unique_attributes + 1) % 10000
    return '_' + `_unique_attributes` + '__'

#
# Return a list of all pairs of objects in a given list
#
def pairs(list):
    p = []
    for i in range(len(list)):
	for j in range(i+1,len(list)):
	    p.append((list[i], list[j]))
    return p

#
# Check if an object represents a well-defined position
#
def isDefinedPosition(p):
    if p is None:
	return 0
    if Numeric.add.reduce(Numeric.greater(p.array, undefined_limit)) > 0:
	return 0
    return 1

#
# Print a warning with reasonable line breaks.
#
def warning(text):
    words = string.split(text)
    text = 'Warning:'
    l = len(text)
    while words:
	lw = len(words[0])
	if l + lw + 1 < 60:
	    text = text + ' ' + words[0]
	    l = l + lw
	else:
	    text = text + '\n' + 9*' ' + words[0]
	    l = lw + 9
	words = words[1:]
    sys.stdout.flush()
    sys.stderr.write(text+"\n")

#
# Pickler and unpickler taking care of non-pickled objects
#
class Pickler(Numeric.Pickler):

    def persistent_id(self, object):
	if hasattr(object, 'is_chemical_object_type'):
	    id = object._restoreId()
	    return id
	else:
	    return None

class Unpickler(Numeric.Unpickler):

    def persistent_load(self, id):
	return eval(id)

#
# General routines for writing objects to files and reading them back
#
def save(object, filename):
    import ChemicalObjects
    filename = os.path.expanduser(filename)
    file = open(filename, 'wb')
    if ChemicalObjects.isChemicalObject(object):
	parent = object.parent
	object.parent = None
	Pickler(file).dump(object)
	object.parent = parent
    else:
	Pickler(file).dump(object)
    file.close()

def load(filename):
    filename = os.path.expanduser(filename)
    file = open(filename, 'rb')
    object = Unpickler(file).load()
    file.close()
    return object

#
# Print information about trajectory file
#
def trajectoryInfo(filename):
    import netcdf
    file = netcdf.NetCDFFile(filename, 'r')
    s = 'Information about trajectory file ' + filename + ':\n'
    try:
	s = s + file.comment + '\n'
    except AttributeError:
	pass
    s = s + `file.dimensions['atom_number']` + ' atoms\n'
    s = s +  `file.variables['step'].shape[0]` + ' steps\n'
    s = s +  file.history
    file.close()
    return s
