/* Private include file for C force field calculations.
 *
 * Written by Konrad Hinsen
 * last revision: 1998-6-29
 */


/* Macro definitions */

#define distance_vector_1(d, r1, r2, data) \
  { \
    d[0] = r2[0]-r1[0]; \
    d[1] = r2[1]-r1[1]; \
    d[2] = r2[2]-r1[2]; \
  }

#define distance_vector_2(d, r1, r2, data) \
  { \
    d[0] = r2[0]-r1[0]; \
    if (d[0] > data[3]) d[0] -= data[0]; \
    if (d[0] < -data[3]) d[0] += data[0]; \
    d[1] = r2[1]-r1[1]; \
    if (d[1] > data[4]) d[1] -= data[1]; \
    if (d[1] < -data[4]) d[1] += data[1]; \
    d[2] = r2[2]-r1[2]; \
    if (d[2] > data[5]) d[2] -= data[2]; \
    if (d[2] < -data[5]) d[2] += data[2]; \
  }

/* Type definitions */

typedef void distance_fn(vector3 d, vector3 r1, vector3 r2, double *data);


/* Functions in _forcefieldmodule.c */

void
distance_vector(vector3 d, vector3 r1, vector3 r2, double *data);

void
orthorhombic_distance_vector(vector3 d, vector3 r1, vector3 r2, double *data);

void
add_pair_fc(PyFFEvaluatorObject *eval, PyObject *fc, fc_function *fc_fn,
	    int i, int j, vector3 dr, double r, double f1, double f2);

void
default_add_energy(PyObject *ev, PyObject *dict);

void
combined_add_energy(PyObject *ev, PyObject *dict);

/* Functions in bonded.c */

ff_eval_function harmonic_bond_evaluator;
ff_eval_function harmonic_angle_evaluator;
ff_eval_function cosine_dihedral_evaluator;


/* Functions in nonbonded.c */

int
nblist_update(PyNonbondedListObject * nblist, int natoms, double *coordinates);

ff_eval_function nonbonded_evaluator;
ff_eval_function lennard_jones_evaluator;
ff_eval_function electrostatic_evaluator;
ff_eval_function es_mp_evaluator;
ff_eval_function lj_es_evaluator;


/* Functions in sparsefc.c */

PyObject *
SparseForceConstants(PyObject *dummy, PyObject *args);

fc_function sparse_fc_function;
