/* Python interface to RasMol library
 *
 * Written by Peter McCluskey
 */
/*
 * probably leaks memory if it raises exception, so don't use it's exceptions
 * for anything other than debugging.
 */

#include "Python.h"
#include "rasapi.h"

static int in_use;
static Molecule __far *Database;
static int labelFullName;
static int pyDrawlabels;

static long
getAtomIndex(PyObject* index_dict, PyObject* a_py)
{
  PyObject* aindex = PyObject_GetItem(index_dict, a_py);
  if(!aindex) return -1;
  Py_DECREF(aindex);
  return PyInt_AsLong(aindex);
}

static void
ReportIndexError(int i, int a_num, int a_index, int num_atoms)
{
  PyObject* err_type = PyExc_IndexError;
  char buf[512];
  sprintf(buf, "Atom bonds[%d].a%d has index %d > number of atoms (%d)",
	  i, a_num, a_index, num_atoms);
  PyErr_SetString(err_type, buf);
}

static int
ConvertAtom(PyObject* atomlist, int index, int num_atoms, PyObject* index_dict,
	    PyObject* number0, PyObject* number1, PyObject* number2)
{
    PyObject* position;
    PyObject* label_py;
    PyObject* label_py2;
    PyObject *x, *y, *z;
    char *name, *long_name;
    PyObject* a_py = PyList_GetItem(atomlist, index);
    PyObject* symbol_py = PyObject_GetAttrString(a_py, "symbol");
    PyObject* pos_args = PyTuple_New(1);
    Atom __far *a = NULL;
    PyObject* py_index = PyInt_FromLong(index);
    if(!pos_args || !py_index)
      return False;
    PyDict_SetItem(index_dict, a_py, py_index);
    Py_DECREF(py_index);
    if(PyTuple_SetItem(pos_args, 0, a_py))
    {
      PyErr_SetString(PyExc_IndexError, "internal error setting pos_args");
      return False;
    }
    position = PyObject_CallMethod(a_py, "position", NULL);
    Py_DECREF(pos_args);
    if(!position)
      return False;
    if(!PyObject_IsTrue(position))
    {
      char buf[512];
      sprintf(buf,"Atom #%d has no position; ignored\n",index);
      WriteString(buf);
      Py_DECREF(position);
      return True;
    }
    a = CreateAtom();
    x = PyObject_GetItem(position, number0);
    if(!x) return False;
    a->xorg = (int)(10000/4*PyFloat_AsDouble(x) + 0.5);
    y = PyObject_GetItem(position, number1);
    if(!y) return False;
    a->yorg = (int)(10000/4*PyFloat_AsDouble(y) + 0.5);
    z = PyObject_GetItem(position, number2);
    if(!z) return False;
    a->zorg = (int)(10000/4*PyFloat_AsDouble(z) + 0.5);
    Py_DECREF(position);
    if(symbol_py)
    {
       char *symbol = PyString_AsString(symbol_py);
       a->refno = SimpleAtomType(symbol);
       Py_DECREF(symbol_py);
    }
    else a->refno = SimpleAtomType("  ");
    a->elemno = GetElemNumberSingle(a);
    label_py2 = PyObject_CallMethod(a_py, "fullName", NULL);
    if(!label_py2) return False;
    label_py = PyObject_GetAttrString(a_py, "name");
    if(!label_py) return False;
    long_name = PyString_AsString(label_py2);
    if(labelFullName)
      name = long_name;
    else name = PyString_AsString(label_py);
    DrawLabels = pyDrawlabels;
    a->label = Create2Labels(name, long_name, strlen(name), strlen(long_name));
    Py_DECREF(label_py);
    Py_DECREF(label_py2);
    a->temp = 0;
    a->serno = index;
    if(index > num_atoms)
    {
      char buf[512];
      sprintf(buf, "Atom %.128s has index %d > number of atoms (%d)",
	      name, index, num_atoms);
      PyErr_SetString(PyExc_IndexError, buf);
      return False;
    }
    return True;
}

static int
CreateDummyAtom(int index, int serno, PyObject* dbondlist,
		PyObject* number0, PyObject* number1, PyObject* number2)
{
  PyObject *x, *y, *z;
  PyObject *position;
  PyObject *label;
  Atom __far *a;
  PyObject* py_index = PyInt_FromLong(index);
  PyObject *dbond = PyObject_GetItem(dbondlist, py_index);
  Py_DECREF(py_index);
  if(!dbond)
    return False;
  position = PyObject_CallMethod(dbond, "DanglingPosition", NULL);
  Py_DECREF(dbond);
  if(!position)
    return False;
  if(!PyObject_IsTrue(position))
  {
    Py_DECREF(position);
    return -1;
  }
  a = CreateAtom();
  x = PyObject_GetItem(position, number0);
  if(!x) return False;
  a->xorg = (int)(10000/4*PyFloat_AsDouble(x) + 0.5);
  y = PyObject_GetItem(position, number1);
  if(!y) return False;
  a->yorg = (int)(10000/4*PyFloat_AsDouble(y) + 0.5);
  z = PyObject_GetItem(position, number2);
  if(!z) return False;
  a->zorg = (int)(10000/4*PyFloat_AsDouble(z) + 0.5);
  Py_DECREF(position);
  a->refno = SimpleAtomType("? ");
  a->elemno = NO_ELEMENT_PRESENT;
  a->label = NULL;
  label = PyObject_CallMethod(dbond, "HookLabel", NULL);
  if(label)
  {
    char *name = PyString_AsString(label);
    Py_DECREF(label);
    if(name)
      a->label = CreateLabel(name, strlen(name));
  }
  a->temp = 0;
  a->serno = serno;
  return True;
}

static int 
ConvertBond(int i,int index,PyObject* bondlist, PyObject* index_dict,
	    int num_atoms)
{
    int is_dangling = (index != i);
    PyObject* i_py = PyInt_FromLong(i);
    PyObject* b = PyObject_GetItem(bondlist, i_py);
    PyObject* order_py = PyObject_GetAttrString(b, "order");
    int order = order_py ? PyInt_AsLong(order_py) : 1;
    int bond_type_flag = NormBondFlag;
    long a1_index,a2_index;
    PyObject* a1 = PyObject_GetAttrString(b, "a1");
    Py_XDECREF(order_py);
    Py_DECREF(i_py);
    if(!a1) return False;
    a1_index = getAtomIndex(index_dict, a1);
    if(is_dangling)
    {
      a2_index = index;
    }
    else
    {
      PyObject* a2 = PyObject_GetAttrString(b, "a2");
      a2_index = getAtomIndex(index_dict, a2);
      Py_DECREF(a2);
    }
    Py_DECREF(a1);
    if(a1_index < 0 || a1_index >= num_atoms)
    {
      ReportIndexError(i, 1, a1_index, num_atoms);
      return False;
    }
    if(a2_index < 0 || a2_index >= num_atoms)
    {
      ReportIndexError(i, 2, a2_index, num_atoms);
      return False;
    }
    if(order >= 2)
      bond_type_flag = DoubBondFlag;
    if(order >= 3)
      bond_type_flag = TripBondFlag;
    CreateBond(a1_index, a2_index, bond_type_flag, Database);
    return True;
}

static int
convertToRasmol(PyObject *pyObj, void *cObj)
{
  int num_atoms;
  int num_bonds;
  int i;
  int ret = 0;
  char buf[512];
  const char* err_msg = NULL;
  PyObject* err_type = PyExc_RuntimeError;
  PyObject *bondlist = NULL;
  PyObject *index_dict = NULL;
  Molecule __far *Database = (Molecule __far *)cObj;
  PyObject* number0 = PyInt_FromLong(0);
  PyObject* number1 = PyInt_FromLong(1);
  PyObject* number2 = PyInt_FromLong(2);
  PyObject *atomlist = PyObject_CallMethod(pyObj, "atomList", NULL);
  if(!atomlist || !PyList_Check(atomlist))
    goto error;
  bondlist = PyObject_CallMethod(pyObj, "bondList", NULL);
  if(bondlist)
  {
    PyObject* bondlist2 = PyObject_GetAttrString(bondlist, "data");
    if(bondlist2)
    {
      Py_DECREF(bondlist);
      bondlist = bondlist2;
    }
  }
  if(!bondlist)
  {
    err_type = PyExc_AttributeError;
    err_msg = "no list of bonds found";
    goto error;
  }
  num_atoms = PyList_Size(atomlist);
  num_bonds = PyList_Size(bondlist);
  index_dict = PyDict_New();
  for(i = 0; i < num_atoms; ++i)
    if(!ConvertAtom(atomlist, i, num_atoms, index_dict, number0, number1, number2))
      goto error;
  for(i = 0; i < num_bonds; ++i)
  {
    if(!ConvertBond(i, i, bondlist, index_dict, num_atoms))
      goto error;
  }
  if(1)
  {
    PyObject *dbondlist = PyObject_CallMethod(pyObj, "danglingBondList", NULL);
    if(dbondlist)
    {
      PyObject *dbondlist2 = PyObject_GetAttrString(dbondlist, "data");
      if(dbondlist2)
      {
	Py_DECREF(dbondlist);
	dbondlist = dbondlist2;
      }
    }
    if(dbondlist)
    {
      int num_dbonds = PyObject_Length(dbondlist);
      for(i = 0; i < num_dbonds; ++i)
      {
	int r = CreateDummyAtom(i,num_atoms + i,dbondlist,number0,number1,number2);
	if(r == -1)
	  continue;
	if(!r)
	  goto error;
	if(!ConvertBond(i,num_atoms + i,dbondlist,index_dict,num_atoms + num_dbonds))
	  goto error;
      }
      Py_DECREF(dbondlist);
    }
  }
  ret = 1;
 error:
  if(err_msg)
    PyErr_SetString(err_type, err_msg);
  Py_XDECREF(atomlist);
  Py_XDECREF(bondlist);
  Py_DECREF(number0);
  Py_DECREF(number1);
  Py_DECREF(number2);
  return ret;
}

static PyObject *
construct(PyObject *self, PyObject *args)
{
  if(in_use)
  {		/* needed due to many global/static vars in RasMol code */
    PyErr_SetString(PyExc_RuntimeError, "RasMol instance cannot be created until the previous instance of a RasMol object is destroyed");
    return NULL;
  }
  in_use = True;
  if(!RasMolInit(NULL))
  {
    PyErr_SetString(PyExc_RuntimeError, "Cannot initialize RasMol");    
    return NULL;
  }
  /* Parse and check arguments */
  Database = CreateMolGroup();
  fprintf(stderr,"construct4\n");
  if (!PyArg_ParseTuple(args, "O&", convertToRasmol, (void*)Database))
    return NULL;
  fprintf(stderr,"construct8\n");
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject*
setLabelFullNameFlag(PyObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, "i", &labelFullName))
    return NULL;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject*
displayLabels(PyObject *self, PyObject *args)
{
  if (!PyArg_ParseTuple(args, "i", &pyDrawlabels))
    return NULL;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *
destruct(PyObject *self, PyObject *args)
{
  RasMolExit();
  in_use = False;
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *
interact(PyObject *self, PyObject *args)
{
  char *command = RasMolInteract(Database);
  if(!command)
  {
    Py_INCREF(Py_None);
    return Py_None;
  }
  return Py_BuildValue("s", command);
}

static PyObject *
refresh(PyObject *self, PyObject *args)
{
  RefreshScreen(Database);
  Py_INCREF(Py_None);
  return Py_None;
}

static int
convertCoordsToC(PyObject *pyObj, void *coordListv)
{
  const char* err_msg = NULL;
  PyObject* err_type = PyExc_RuntimeError;
  int good = 1;
  char buf[512];
  Poly **coordListPtr = (Poly **)coordListv;
  Poly *poly;
  PyObject* number0 = PyInt_FromLong(0);
  PyObject* number1 = PyInt_FromLong(1);
  PyObject* number2 = PyInt_FromLong(2);
  int n = PyObject_Length(pyObj);
  poly = RasGetNewPoly(n);
  if(!poly)
  {
    err_type = PyExc_ValueError;
    err_msg = "Unable to allocate RasMol Poly - too large points or too many polygons";
    good = 0;
  }
  else
  {
    int i;
    for(i = 0; i < n; ++i)
    {
      PyObject* numberi = PyInt_FromLong(i);
      PyObject* v = PyObject_GetItem(pyObj, numberi);
      Py_DECREF(numberi);
      if(!v)
      {
	err_type = PyExc_TypeError;
	err_msg = "?";
	good = 0;
      }
      else
      {
	PyObject* v0 = PyObject_GetItem(v, number0);
	PyObject* v1 = PyObject_GetItem(v, number1);
	PyObject* v2 = PyObject_GetItem(v, number2);
	if(v0 && v1 && v2)
	{
	  double x = PyFloat_AsDouble(v0);
	  double y = PyFloat_AsDouble(v1);
	  double z = PyFloat_AsDouble(v2);
	  RasSetPolyVertex(&poly->v[i], (int)(10000/4*x + 0.5),
			   (int)(10000/4*y + 0.5), (int)(10000/4*z + 0.5));
	}
	else
	{
	  good = 0;
	  err_type = PyExc_IndexError;
	  sprintf(buf, "__getitem__ failed on coordinate %d", i);
	  err_msg = buf;
	}
      }
    }
    *coordListPtr = poly;
  }
 error:
  if(err_msg)
    PyErr_SetString(err_type, err_msg);
  Py_DECREF(number0);
  Py_DECREF(number1);
  Py_DECREF(number2);
  return good;
}

#define ForEachAtom  for(chain=Database->clist;chain;chain=chain->cnext) \
		     for(group=chain->glist;group;group=group->gnext)    \
		     for(ptr=group->alist;ptr;ptr=ptr->anext)

static PyObject *
SelectAtoms(PyObject *self, PyObject *args)
{
  register Chain __far *chain;
  register Group __far *group;
  register Atom __far *ptr;
  PyObject* select_atoms;
  int SelectCount = 0;
  if (!PyArg_ParseTuple(args, "O", &select_atoms))
    return NULL;
  ForEachAtom
  {
    PyObject* py_index = PyInt_FromLong((long)ptr->serno);
    if(PyMapping_HasKey(select_atoms, py_index))
    {
      ptr->flag |= SelectFlag;
      SelectCount++;
    }
    else
      ptr->flag &= ~SelectFlag;
    Py_DECREF(py_index);
  }
  return Py_BuildValue("l", (long)SelectCount);
}

#define ForEachBond  for(bptr=Database->blist;bptr;bptr=bptr->bnext) 

static PyObject *
SelectBonds(PyObject *self, PyObject *args)
{
  register Bond __far *bptr;
  PyObject* select_bonds;
  int SelectCount = 0;
  if (!PyArg_ParseTuple(args, "O", &select_bonds))
    return NULL;
  ForEachBond
  {
    PyObject* py_key = PyTuple_New(2);
    PyObject* index1 = PyInt_FromLong((long)bptr->dstatom->serno);
    PyObject* index2 = PyInt_FromLong((long)bptr->srcatom->serno);
    PyTuple_SetItem(py_key, 0, index1);
    PyTuple_SetItem(py_key, 1, index2);
    if(PyMapping_HasKey(select_bonds, py_key))
    {
      bptr->flag |= DrawBondFlag;
      SelectCount++;
    }
    else
      bptr->flag &= ~DrawBondFlag;
    Py_DECREF(py_key);
    Py_DECREF(index1);
    Py_DECREF(index2);
  }
  return Py_BuildValue("l", (long)SelectCount);
}

static PyObject *
AddPolygon(PyObject *self, PyObject *args)
{
  Poly* coordList;
  if (!PyArg_ParseTuple(args, "O&", convertCoordsToC, &coordList))
    return NULL;
  RasMolAddPolygon(coordList);
  return Py_BuildValue("l", (long)coordList);
}

static PyObject *
AddPolyLines(PyObject *self, PyObject *args)
{
  Poly* coordList;
  if (!PyArg_ParseTuple(args, "O&", convertCoordsToC, &coordList))
    return NULL;
  RasMolAddPolyLines(coordList);
  return Py_BuildValue("l", (long)coordList);
}

static PyObject *
SetPolyColor(PyObject *self, PyObject *args)
{
  PyObject* colors;
  if (!PyArg_ParseTuple(args, "O", &colors))
    return NULL;
  else
  {
    PyObject* number0 = PyInt_FromLong(0);
    PyObject* number1 = PyInt_FromLong(1);
    PyObject* number2 = PyInt_FromLong(2);
    PyObject* rpy = PyObject_GetItem(colors, number0);
    PyObject* gpy = PyObject_GetItem(colors, number1);
    PyObject* bpy = PyObject_GetItem(colors, number2);
    Py_DECREF(number0);
    Py_DECREF(number1);
    Py_DECREF(number2);
    if(rpy && gpy && bpy)
    {
	  RasMolSetPolyColor((Byte)PyInt_AsLong(rpy), (Byte)PyInt_AsLong(gpy),
			     (Byte)PyInt_AsLong(bpy));
    }
  }
  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject *
MaxPolyCoords(PyObject *self, PyObject *args)
{
  return Py_BuildValue("l", (long)MAXVERT);
}

static PyObject *
RemovePoly(PyObject *self, PyObject *args)
{
  long coordList;
  if (!PyArg_ParseTuple(args, "l", &coordList))
    return NULL;
  RasMolRemovePoly((Poly *)coordList);
  Py_INCREF(Py_None);
  return Py_None;
}


/*
 * List of functions defined in the module
 */

static PyMethodDef rasmol_methods[] = {
  {"__init__", construct, METH_VARARGS},
  {"__del__", destruct, METH_VARARGS},
  {"setLabelFullNameFlag",setLabelFullNameFlag,METH_VARARGS},
  {"displayLabels",displayLabels,METH_VARARGS},
  {"RefreshScreen", refresh, METH_VARARGS},
  {"Interact", interact, METH_VARARGS},
  {"SelectAtoms", SelectAtoms, METH_VARARGS},
  {"SelectBonds", SelectBonds, METH_VARARGS},
  {"AddPolygon", AddPolygon, METH_VARARGS},
  {"AddPolyLines", AddPolyLines, METH_VARARGS},
  {"RemovePoly", RemovePoly, METH_VARARGS},
  {"MaxPolyCoords", MaxPolyCoords, METH_VARARGS},
  {"SetPolyColor", SetPolyColor, METH_VARARGS},
  /*{"", update_positions, METH_VARARGS}, */
  {NULL, NULL}		/* sentinel */
};

/* Initialization function for the module */

void
init_rasmol()
{
  PyObject *m;

  /* Create the module and add the functions */
  m = Py_InitModule("_rasmol", rasmol_methods);
  
  /* Check for errors */
  if (PyErr_Occurred())
    Py_FatalError("can't initialize module rasmol");
  RasSetQLabels(True);
}
